@@ -1,8 +1,8 @@
 # Net::CIDR
 #
-# Copyright 2001-2019 Sam Varshavchik.
+# Copyright 2001-2025 Sam Varshavchik.
 #
-# with contributions from David Cantrell.
+# with contributions from David Cantrell and brian d foy
 #
 # This program is free software; you can redistribute it
 # and/or modify it under the same terms as Perl itself.
@@ -50,7 +50,7 @@ use Carp;
 
 );
 
-$VERSION = "0.20";
+$VERSION = "0.25";
 
 1;
 
@@ -67,6 +67,13 @@ Net::CIDR - Manipulate IPv4/IPv6 netblocks in CIDR notation
 
     use Net::CIDR ':all';
 
+    my $var;
+
+    if ($var = Net::CIDR::cidrvalidate($var))
+    {
+         // ... do something
+    }
+
     print join("\n",
           Net::CIDR::range2cidr("192.168.0.0-192.168.255.255",
 		                "10.0.0.0-10.3.255.255"))
@@ -171,6 +178,12 @@ The Net::CIDR package contains functions that manipulate lists of IP
 netblocks expressed in CIDR notation.
 The Net::CIDR functions handle both IPv4 and IPv6 addresses.
 
+The cidrvalidate() function, described below, checks that its argument
+is a single, valid IP address or a CIDR. The remaining functions
+expect that
+their parameters consist of validated IPs or CIDRs. See cidrvalidate()
+and BUGS, below, for more information.
+
 =head2 @cidr_list=Net::CIDR::range2cidr(@range_list);
 
 Each element in the @range_list is a string "start-finish", where
@@ -523,7 +536,7 @@ sub addr2cidr {
 	    $n &= 0xF8;
 	}
 
-	my $s=join(".", @ipcpy);
+	my $s=join(".", map { s/\A0+([0-9])/$1/; $_ } @ipcpy);
 
 	push @blocks, ($isipv6 ? _ipv4to6($s):$s) . "/$bits";
     }
@@ -859,7 +872,7 @@ sub cidr2octets {
 
 	$cidr =~ s/\s//g;
 
-	croak "CIDR doesn't look like a CIDR\n" unless ($cidr =~ /(.*)\/(.*)/);
+	croak "CIDR \"$cidr\" doesn't look like a CIDR\n" unless ($cidr =~ /(.*)\/(.*)/);
 
 	my ($ip, $pfix)=($1, $2);
 
@@ -997,7 +1010,7 @@ sub cidradd {
     my @b;
 
     grep {
-	croak "This doesn't look like start-end\n" unless /(.*)-(.*)/;
+	croak "Range $_ doesn't look like start-end\n" unless /(.*)-(.*)/;
 	push @a, $1;
 	push @b, $2;
     } @cidr;
@@ -1165,7 +1178,7 @@ sub cidrlookup {
     my @b;
 
     grep {
-	croak "This doesn't look like start-end\n" unless /(.*)-(.*)/;
+	croak "Range $_ doesn't look like start-end\n" unless /(.*)-(.*)/;
 	push @a, $1;
 	push @b, $2;
     } @cidr;
@@ -1205,6 +1218,29 @@ undef.
 
 =cut
 
+sub _compress_ipv6 {
+    # taken from IPv6::Address on CPAN
+    my $str = shift;
+    return '::' if($str eq '0:0:0:0:0:0:0:0');
+    for(my $i=7;$i>1;$i--) {
+            my $zerostr = join(':',split('','0'x$i));
+            ###print "DEBUG: $str $zerostr \n";
+            if($str =~ /:$zerostr$/) {
+                    $str =~ s/:$zerostr$/::/;
+                    return $str;
+            }
+            elsif ($str =~ /:$zerostr:/) {
+                    $str =~ s/:$zerostr:/::/;
+                    return $str;
+            }
+            elsif ($str =~ /^$zerostr:/) {
+                    $str =~ s/^$zerostr:/::/;
+                    return $str;
+            }
+    }
+    return $str;
+}
+
 sub cidrvalidate {
     my $v=shift;
 
@@ -1284,9 +1320,10 @@ sub cidrvalidate {
 
     $v =~ s/([0-9A-Fa-f]+)/_triml0($1)/ge;
 
+	my $compressed = _compress_ipv6($v);
     foreach (addr2cidr($v))
     {
-	return $_ if $_ eq "$v/$suffix";
+	return $_ if $_ eq "$compressed/$suffix";
     }
     return undef;
 }
@@ -1315,6 +1352,7 @@ but the functions will croak if you're totally off the wall.
 Sam Varshavchik <sam@email-scan.com>
 
 With some contributions from David Cantrell <david@cantrell.org.uk>
+and brian d foy <briandfoy@pobox.com>.
 
 =cut
 
